package edu.lawrence.androidgrades

import androidx.room.Dao
import androidx.room.Database
import androidx.room.Delete
import androidx.room.Entity
import androidx.room.Insert
import androidx.room.OnConflictStrategy
import androidx.room.PrimaryKey
import androidx.room.Query
import androidx.room.RoomDatabase

@Entity
data class Student(@PrimaryKey(autoGenerate=true) val id : Int = 0, val firstName : String, val lastName : String)

@Entity
data class Quiz(@PrimaryKey(autoGenerate=true) val id : Int = 0, val maxPoints : Int, val name : String)

@Entity
data class Grade(@PrimaryKey(autoGenerate=true) val id : Int = 0, val student : Int, val quiz : Int, val grade : Int)

@Dao
interface GradesDao {
    @Insert(onConflict = OnConflictStrategy.REPLACE)
    suspend fun insertStudent(student : Student)

    @Insert(onConflict = OnConflictStrategy.REPLACE)
    suspend fun insertQuiz(quiz : Quiz)

    @Insert(onConflict = OnConflictStrategy.REPLACE)
    suspend fun insertGrade(grade : Grade)

    @Delete
    suspend fun removeGrade(grade : Grade)

    @Query("SELECT * from student ORDER BY lastName ASC, firstName ASC")
    suspend fun loadStudents() : List<Student>

    @Query("SELECT * from quiz")
    suspend fun loadQuizzes() : List<Quiz>

    @Query("SELECT * from grade WHERE student = :student and quiz = :quiz")
    suspend fun getGrade(student: Int, quiz : Int) : List<Grade>
}

@Database(entities = [Student::class,Quiz::class,Grade::class], version = 1)
abstract class GradesDatabase : RoomDatabase() {
    abstract fun gradesDao() : GradesDao
}

