package edu.lawrence.androidgrades

import android.app.Application
import androidx.compose.runtime.mutableStateOf
import androidx.lifecycle.AndroidViewModel
import androidx.lifecycle.viewModelScope
import androidx.room.Room
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

class GradesModel(application: Application): AndroidViewModel(application) {
    lateinit var dao : GradesDao
    val students = mutableStateOf<List<Student>?>(null)
    val firstStudent = mutableStateOf<Student?>(null)
    val quizzes = mutableStateOf<List<Quiz>?>(null)
    val firstQuiz = mutableStateOf<Quiz?>(null)
    val grade = mutableStateOf<Grade?>(null)

    init {
        val db = Room.databaseBuilder(
            application,
            GradesDatabase::class.java, "grades"
        ).build()
        dao = db.gradesDao()

        viewModelScope.launch {
            var studentList : List<Student>
            withContext(Dispatchers.IO) {
                studentList = dao.loadStudents()
            }
            withContext(Dispatchers.Main) {
                if(!studentList.isEmpty())
                    firstStudent.value = studentList[0]
                students.value = studentList
            }
            var quizList : List<Quiz>
            withContext(Dispatchers.IO) {
                quizList = dao.loadQuizzes()
            }
            withContext(Dispatchers.Main) {
                if(!quizList.isEmpty())
                    firstQuiz.value = quizList[0]
                quizzes.value = quizList
            }
        }
    }

    fun addStudent(student: Student) {
        viewModelScope.launch {
            var studentList: List<Student>
            withContext(Dispatchers.IO) {
                dao.insertStudent(student)
                studentList = dao.loadStudents()
            }
            withContext(Dispatchers.Main) {
                firstStudent.value = studentList[0]
                students.value = studentList
            }
        }
    }

    fun addQuiz(quiz : Quiz) {
        viewModelScope.launch {
            var quizList: List<Quiz>
            withContext(Dispatchers.IO) {
                dao.insertQuiz(quiz)
                quizList = dao.loadQuizzes()
            }
            withContext(Dispatchers.Main) {
                firstQuiz.value = quizList[0]
                quizzes.value = quizList
            }
        }
    }

    fun initGrade() {
        if(firstStudent.value != null && firstQuiz.value != null)
            getGrade(firstStudent.value!!,firstQuiz.value!!)
        else
            grade.value = null
    }

    fun addGrade(newGrade : Grade) {
        viewModelScope.launch {
            var grades: List<Grade>
            withContext(Dispatchers.IO) {
                grades = dao.getGrade(
                    student = newGrade.student,
                    quiz = newGrade.quiz
                )
                if(!grades.isEmpty())
                    dao.removeGrade(grades[0])
                dao.insertGrade(newGrade)
            }
            withContext(Dispatchers.Main) {
                grade.value = newGrade
            }
        }
    }

    fun getGrade(student : Student,quiz : Quiz) {
        viewModelScope.launch {
            var grades: List<Grade>
            withContext(Dispatchers.IO) {
                grades = dao.getGrade(
                    student = student.id,
                    quiz = quiz.id
                )
            }
            withContext(Dispatchers.Main) {
                if (!grades.isEmpty()) {
                    grade.value = grades[0]
                } else {
                    grade.value = null
                }
            }
        }
    }
}